@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Config.pm,v 1.81.2.65 2004/12/18 20:56:36 jkf Exp $
+#   $Id: Config.pm,v 1.81.2.71 2005/03/07 21:01:26 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.81.2.65 $, 10;
+$VERSION = substr q$Revision: 1.81.2.71 $, 10;
 
 #1 exception to general rule of forming replies to Config::Value()
 #is %Config::ScannerCmds
@@ -67,6 +67,7 @@ my(%RuleScalars, %Defaults, $DefaultAddressRegexp, $DefaultVirusRegexp);
 my(%CustomFunctions, %CustomFunctionsParams);
 my(%PercentVars); # For putting substituted variables in all settings
 my($RequireLDAPDone);
+use vars qw(%PhishingWhitelist); # Whitelist of hostnames for Phishing Net
 use vars qw($LDAP $LDAPserver $LDAPbase $LDAPsite); # LDAP connection info
 
 $RequireLDAPDone = 0; # Have we done the "require Net::LDAP"?
@@ -346,7 +347,15 @@ sub FirstMatchValue {
       # Can only check these with From:, not To: addresses
       # Match against the SMTP Client IP address
       #print STDERR "Matching IP " . $msg->{clientip} . " against $regexp\n";
-      return $value if $msg->{clientip} =~ /$regexp/;
+      if ($regexp =~ /\d+\\\.\d+\\\.\d+\\\.\d+\)*$/) {
+        # It's a complete IPv4 address so it's a total string match, not a re
+        #print STDERR "Got a match\n";
+        return $value if $msg->{clientip} =~ /^$regexp$/;
+      } else {
+        # It's not a complete IPv4 address so substring match it
+        #print STDERR "Got no match\n";
+        return $value if $msg->{clientip} =~ /$regexp/;
+      }
     }
     if ($direction =~ /[tb]/) {
       # Don't know the target IP address
@@ -459,7 +468,13 @@ sub AllMatchesValue {
       # It's a numeric ip-number-based rule
       # Can only check these with From:, not To: addresses
       # Match against the SMTP Client IP address
-      push @matches, split(" ",$value) if $msg->{clientip} =~ /$regexp/;
+      if ($regexp =~ /\d+\\\.\d+\\\.\d+\\\.\d+\)*$/) {
+        # It's a complete IPv4 address so it's a total string match, not a re
+        push @matches, split(" ",$value) if $msg->{clientip} =~ /^$regexp$/;
+      } else {
+        # It's not a complete IPv4 address so substring match it
+        push @matches, split(" ",$value) if $msg->{clientip} =~ /$regexp/;
+      }
     } else {
       # Don't know the target IP address
       MailScanner::Log::WarnLog("Cannot match against destination " .
@@ -545,7 +560,7 @@ sub initialise {
 
   while(defined($filename = $dirh->read)) {
     # Skip all dot files and rpmnew files
-    next if $filename =~ /^\./ || $filename =~ /\.rpmnew$/i;
+    next if $filename =~ /^\./ || $filename =~ /\.(rpmnew|dpkg-dist|dpkg-new|dpkg-old)$/i;
     $fullfile = "$dir/$filename";
     next unless -f $fullfile and -s $fullfile;
     eval { require $fullfile; };
@@ -589,8 +604,9 @@ sub QuickPeek {
     /^(.*?)\s*=\s*(.*)$/;
     ($key,$value) = ($1,$2);
 
-    # Allow %var% = value lines
+    # Allow %var% = value lines with $VAR in value
     $value =~ s/\%([^%]+)\%/$PercentVars{lc($1)}/g;
+    $value =~ s/\$\{?(\w+)\}?/$ENV{$1}/g;
     if ($key =~ /^\%([^%]+)\%$/) {
       $PercentVars{lc($1)} = $value;
       next;
@@ -758,6 +774,10 @@ sub Read {
   # These map a spam list name onto a DNS domain
   %SpamLists = ReadDefinitions(Value('spamlistdefinitions'));
 
+  # Read in the Phishing Net whitelist.
+  # This lists all the hostnames of places to ignore when phishing.
+  %PhishingWhitelist = ReadPhishingWhitelist(Value('phishingwhitelist'));
+
   # Call all the user's custom initialisation functions
   my($key, $param, $custom, $fn);
   foreach $key (keys %CustomFunctions) {
@@ -787,6 +807,47 @@ sub Read {
   $Defaults{'enablespambounce'} = 0;
 }
 
+# Read the list of hostnames to be ignored when doing phishing tests.
+# Pass in the filename. Return the hash.
+sub ReadPhishingWhitelist {
+  my($filename) = @_;
+
+  my($fh, %whitelist, $counter);
+
+  %whitelist = ();
+
+  # Skip this if they have findphishing = no
+  return if MailScanner::Config::IsSimpleValue('findphishing') &&
+            !MailScanner::Config::Value('findphishing');
+
+  $filename =~ s/^\s*//g;
+  $filename =~ s/\s*$//g;
+  return () unless $filename;
+
+  $fh = new FileHandle;
+  unless (open($fh, "<$filename")) {
+    MailScanner::Log::WarnLog("Could not read phishing whitelist file %s");
+    return ();
+  }
+
+  $counter = 0;
+  while(<$fh>) {
+    chomp;
+    s/^#.*$//;   # Remove comments
+    s/^\s*//g;   # Remove leading white space
+    s/\s*$//g;   # Remove trailing white space
+    s/\s+.*$//g; # Leave only the 1st word
+    next if /^$/;
+    $whitelist{$_} = 1;
+    $counter++;
+  }
+
+  close $fh;
+  MailScanner::Log::InfoLog("Read %d hostnames from the phishing whitelist",
+                            $counter);
+
+  return %whitelist;
+}
 
 # Give all the user's custom functions a chance to clear up
 # and neatly shutdown, log totals, close databases, etc.
@@ -1277,6 +1338,7 @@ sub ReadInQueueDirs {
   $taintedname =~ /(.*)/;
   $name = $1;
   $name =~ s/\%([^%]+)\%/$PercentVars{lc($1)}/g;
+  $name =~ s/\$\{?(\w+)\}?/$ENV{$1}/g;
 
   if ($name =~ /[\?\*]/) {
     # It's a glob so contains directory names
@@ -1319,6 +1381,7 @@ sub ReadInQueueDirs {
     next if $1 eq "";
     $dir = $1;
     $dir =~ s/\%([^%]+)\%/$PercentVars{lc($1)}/g;
+    $dir =~ s/\$\{?(\w+)\}?/$ENV{$1}/g;
     if ($dir =~ /[\?\*]/) {
       # It's a glob so contains directory names
       @list = glob($dir);
@@ -1501,6 +1564,7 @@ sub ReadConfFile {
 
     # Allow %var% = value lines
     $value =~ s/\%([^%]+)\%/$PercentVars{lc($1)}/g;
+    $value =~ s/\$\{?(\w+)\}?/$ENV{$1}/g;
     if ($origkey =~ /^\%([^%]+)\%$/) {
       $PercentVars{lc($1)} = $value;
       next;
@@ -2063,6 +2127,7 @@ sub Store1Rule {
 
   # Substitute %% variables
   $value =~ s/\%([^%]+)\%/$PercentVars{lc($1)}/g;
+  $value =~ s/\$\{?(\w+)\}?/$ENV{$1}/g;
 
   # Syntax check the value
   #print STDERR "Config: $keyword has rule value " . $File{$keyword} .
@@ -2636,6 +2701,7 @@ sub ReadOtherValue {
       #             " = $internal\n";
       # Percent variables need to be substituted here.
       $internal = DoPercentVars($internal); # %vars% must be defined before use
+      $internal =~ s/\$\{?(\w+)\}?/$ENV{$1}/g;
       $StaticScalars{$keyword} = $internal;
     }
     # Could do some specific syntax checking in here
