@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl,v 1.37.2.77 2004/12/18 12:25:43 jkf Exp $
+#   $Id: ConfigDefs.pl,v 1.37.2.84 2005/03/05 19:44:39 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -65,6 +65,10 @@ BlockEncrypted			= BlockEncryptedMessages
 BlockUnencrypted		= BlockUnencryptedMessages
 CheckSAIfOnSpamList		= checkspamassassinifonspamlist
 children			= maxchildren
+clamavmaxreclevel		= clamavmodulemaximumrecursionlevel
+clamavmaxfiles			= clamavmodulemaximumfiles
+clamavmaxfilesize		= clamavmodulemaximumfilesize
+clamavmaxratio			= clamavmodulemaximumcompressionratio
 clamwatchfiles			= monitorsforclamavupdates
 cleanheader			= cleanheadervalue
 contentprependsubject		= contentmodifysubject
@@ -105,6 +109,7 @@ inqueuedir			= incomingqueuedir
 keepspamarchiveclean		= keepspamandmcparchiveclean
 lastlookup			= alwayslookeduplast
 listsascores                    = includescoresinspamassassinreport
+loghtmltags			= logdangeroushtmltags
 logfacility			= syslogfacility
 maxattachmentsize		= maximumattachmentsize
 maxdirtybytes			= maxunsafebytesperscan
@@ -122,6 +127,8 @@ noisyviruses			= nonforgingviruses
 nosenderprecedence		= nevernotifysendersofprecedence
 noticefullheaders		= noticesincludefullheaders
 noticerecipient			= noticesto
+phishingnumbers			= alsofindnumericphishing
+phishingwhitelist		= phishingsafesitesfile
 outqueuedir			= outgoingqueuedir
 quarantinesilent		= quarantinesilentviruses
 quarantineuser			= quarantineuser
@@ -215,6 +222,7 @@ lockfiledir		/tmp
 
 # Check the first word of these for file existence
 [Simple,File]
+PhishingWhitelist	/opt/MailScanner/etc/phishing.safe.sites.conf
 pidfile			/var/run/MailScanner.pid
 spamassassinprefsfile	/opt/MailScanner/etc/spam.assassin.prefs.conf
 SpamListDefinitions	/opt/MailScanner/etc/spam.lists.conf
@@ -225,6 +233,10 @@ VirusScannerDefinitions	/opt/MailScanner/etc/virus.scanners.conf
 [Simple,Number]
 BayesRebuild			0
 Children			1
+clamavmaxreclevel               5
+clamavmaxfiles                  1000
+clamavmaxfilesize               10000000
+clamavmaxratio                  250
 CriticalQueueSize		5000
 FileTimeout			30
 MaxUnscannedBytes		100000000
@@ -244,6 +256,7 @@ MCPMaxSpamAssassinTimeouts	20
 MCPMaxSpamAssassinSize		100000
 MCPSpamAssassinTimeout		10
 TNEFTimeout			120
+UnrarTimeout			50
 WhitelistMaxRecips		20
 
 # These are all the other strings I haven't categorised.
@@ -284,6 +297,7 @@ MCPSpamAssassinLocalRulesDir	/opt/MailScanner/etc/mcp
 MCPSpamAssassinDefaultRulesDir	/opt/MailScanner/etc/mcp
 MCPSpamAssassinInstallPrefix	/opt/MailScanner/etc/mcp
 TNEFExpander		/opt/MailScanner/bin/tnef --maxsize=100000000
+UnrarCommand		/usr/bin/unrar
 VirusScanners		none  # Space-separated list
 WorkUser
 WorkGroup
@@ -318,7 +332,7 @@ KeepSpamArchiveClean	1	no	0	yes	1
 LastLookup		0	no	0	yes	1
 ListSAScores		0	no	0	yes	1
 #LoadSpamAssassin	0	no	0	yes	1
-LogIFrameTags		0	no	0	yes	1
+LogHTMLTags		0	no	0	yes	1
 LogPermittedFilenames	0	no	0	yes	1
 LogPermittedFiletypes	0	no	0	yes	1
 MultipleHeaders		append	append	append	replace	replace	add	add
@@ -451,6 +465,7 @@ DeliverCleanedMessages	1	no	0	yes	1
 FindPhishing		0	no	0	yes	1
 markinfectedmessages	1	no	0	yes	1
 HtmlToText		0	no	0	yes	1
+PhishingNumbers		1	no	0	yes	1
 QuarantineInfections	1	no	0	yes	1
 QuarantineSilent	1	no	0	yes	1
 QuarantineWholeMessage	0	no	0	yes	1
