@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Exim.pm,v 1.24.2.34 2004/12/18 12:25:44 jkf Exp $
+#   $Id: Exim.pm,v 1.24.2.40 2005/03/27 18:21:32 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ use IO::Pipe;
 use Carp;
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.24.2.34 $, 10;
+$VERSION = substr q$Revision: 1.24.2.40 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process
 # a message that is piped to it. Still need to add the envelope sender
@@ -552,6 +552,9 @@ sub ReadQf {
   # onto the metadata headers array at this point...
   exists $header->{name} and push @{$metadata{headers}},$header;
 
+  # Decode the ISO encoded Subject line
+  $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
+
   # I'd prefer that $message->{headers} not exist;
   # it's an incitement to do bad things that defeat
   # the point of hiding the internal implementation
@@ -560,7 +563,7 @@ sub ReadQf {
   $message->{headers} = \@headers;
   $message->{metadata} = \%metadata;
 
-#  print Dumper($message->{headers});
+  #print STDERR Dumper($message->{metadata});
   return 1;
 }
 
@@ -574,7 +577,7 @@ sub AddHeadersToQf {
   
   my($header, $h, @newheaders);
   
-#  print Dumper($message->{headers});
+  #print STDERR Dumper($message->{headers});
 
   if (defined $headers) {
     @newheaders = split(/\n/, $headers);
@@ -751,6 +754,25 @@ sub DeleteHeader {
   }
 }
 
+sub UniqHeader {
+  my($this, $message, $key) = @_;
+
+  my $hdrnum;
+  my $foundat = -1;
+  my $metadata = $message->{metadata};
+  for ($hdrnum=0; $hdrnum<@{$metadata->{headers}}; $hdrnum++) {
+    next unless lc $metadata->{headers}[$hdrnum]{name} eq lc $key;
+
+    # Have found the header line, skip it if we haven't seen it before
+    ($foundat = $hdrnum), next if $foundat == -1;
+
+    # Have found the right line
+    $metadata->{headers}[$hdrnum]{flag} ne " "
+      and $metadata->{vanishedflags}{$metadata->{headers}[$hdrnum]{flag}} = 1;
+    $metadata->{headers}[$hdrnum]{flag} = "*";
+  }
+}
+
 
 # We need to delete *all* instances of the header in
 # question, as this is used e.g. to replace previous
@@ -965,7 +987,7 @@ sub DeleteRecipients {
 sub KickMessage {
   my $pid;
 
-  # My original implementation
+  ## My original implementation
   #my(@ids) = @_;
   #my($idlist);
   #$idlist = join(' ', @ids);
@@ -973,6 +995,7 @@ sub KickMessage {
   ##print STDERR "About to do \"Sendmail2 -Mc $idlist\"\n";
   #system(MailScanner::Config::Value('sendmail2') . ' -Mc ' . $idlist);
 
+  # JJH's version
   if(MailScanner::Config::Value('deliverinbackground')) {
     # fork twice so that we don't have to reap :-)
     $pid = fork;
@@ -1420,7 +1443,8 @@ sub CreateBatch {
       $batch->{messages}{"$id"} = $newmessage;
       $batchempty = 0;
 
-      if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/) {
+      if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+          MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
 	$newmessage->NeedsScanning(1);
 	$DirtyMsgs++;
 	$DirtyBytes += $newmessage->{size};
