@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Log.pm,v 1.14.2.7 2004/11/28 11:26:51 jkf Exp $
+#   $Id: Log.pm,v 1.14.2.8 2005/02/18 14:08:30 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -135,6 +135,9 @@ sub LogText {
   #my $old = $SIG{'PIPE'};
   #$SIG{'PIPE'} = sub { $MailScanner::Log::SIGPIPE_RECEIVED++; };
 
+  # Force use of 8-bit characters, UTF16 breaks syslog badly.
+  use bytes;
+
   foreach(split /\n/,$logmessage) {
     s/%/%%/g;
     Sys::Syslog::syslog($level, $_) if $_ ne "";
