@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MCP.pm,v 1.1.2.5 2004/04/11 17:30:00 jkf Exp $
+#   $Id: MCP.pm,v 1.1.2.8 2004/10/16 14:23:14 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use IO;
 use vars qw($VERSION $SAspamtest $SABayesLock);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.5 $, 10;
+$VERSION = substr q$Revision: 1.1.2.8 $, 10;
 
 # Attributes are
 #
@@ -221,10 +221,16 @@ sub Checks {
   #foreach $h (@{$message->{headers}}) {
   #  push @WholeMessage, $h . "\n";
   #}
+  my $fromheader = MailScanner::Config::Value('envfromheader', $message);
+  $fromheader =~ s/:$//;
+  push(@WholeMessage, $fromheader . ': ' . $message->{from} . "\n")
+    if $fromheader;
+
   @WholeMessage = $global::MS->{mta}->OriginalMsgHeaders($message, "\n");
   #print STDERR "Headers are : " . join(', ', @WholeMessage) . "\n";
   return (0,0, MailScanner::Config::LanguageValue($message, 'mcpsanoheaders'), 0)
     unless @WholeMessage;
+
   push(@WholeMessage, "\n");
   $message->{store}->ReadBody(\@WholeMessage, $maxsize);
 
