@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MCPMessage.pm,v 1.1.2.16 2004/12/20 14:57:14 jkf Exp $
+#   $Id: MCPMessage.pm,v 1.1.2.22 2005/04/29 21:23:24 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ install MIME::Decoder::UU 'uuencode';
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.16 $, 10;
+$VERSION = substr q$Revision: 1.1.2.22 $, 10;
 
 # Is this message spam? Try to build the spam report and store it in
 # the message.
@@ -76,7 +76,7 @@ sub IsMCP {
   $this->{ismcp} = 0;
   $this->{ishighmcp} = 0;
   $this->{mcpreport} = "";
-  $this->{mcpscore} = 0;
+  $this->{mcpsascore} = 0;
 
   ## If it's a blacklisted address, don't bother doing any checks at all
   #if (MailScanner::Config::Value('spamblacklist', $this)) {
@@ -110,7 +110,7 @@ sub IsMCP {
     $this->{ismcp} = 1;
     $this->{ishighmcp} = 1
       if MailScanner::Config::Value('mcpblacklistedishigh', $this);
-    $this->{spamreport} = $LocalSpamText . ' (' .
+    $this->{mcpreport} = $LocalSpamText . ' (' .
                    MailScanner::Config::LanguageValue($this, 'mcpblacklisted') .
                    ')';
     MailScanner::Log::InfoLog("Message %s from %s (%s) to %s" .
@@ -171,7 +171,7 @@ sub IsMCP {
   my $sascore  = 0;
   ($SAsaysspam, $SAHighScoring, $saheader, $sascore)
     = MailScanner::MCP::Checks($this);
-  $this->{mcpscore} = $sascore; # Save the actual figure for use later...
+  $this->{mcpsascore} = $sascore; # Save the actual figure for use later...
 
   # Fix the return values
   $SAsaysspam = 0 unless $saheader;    # Solve bug with empty SAreports
@@ -369,7 +369,8 @@ sub HandleMCP {
   if ($actions{'store'}) {
     my($dir, $dir2, $spamdir, $uid, $gid, $changeowner);
     $dir = MailScanner::Config::Value('quarantinedir', $this);
-    $dir2 = $dir . '/' .  MailScanner::Quarantine::TodayDir();
+    #$dir2 = $dir . '/' .  MailScanner::Quarantine::TodayDir();
+    $dir2 = $dir . '/' .  $this->{datenumber};
     $spamdir = $dir2 . '/' . $HamSpam;
     $uid = $global::MS->{quar}->{uid};
     $gid = $global::MS->{quar}->{gid};
@@ -421,6 +422,7 @@ sub HandleMCPBounce {
 
   my($from,$to,$subject,$date,$spamreport,$hostname);
   my($emailmsg, $line, $messagefh, $filename, $localpostmaster, $id);
+  my($postmastername);
 
   $from = $this->{from};
 
@@ -451,10 +453,11 @@ sub HandleMCPBounce {
   $id = $this->{id};
   $to = join(', ', @{$this->{to}});
   $localpostmaster = MailScanner::Config::Value('localpostmaster', $this);
+  $postmastername  = MailScanner::Config::LanguageValue($this, 'mailscanner');
   $hostname = MailScanner::Config::Value('hostname', $this);
   $subject = $this->{subject};
-  $date = scalar localtime;
-  $spamreport = $this->{spamreport};
+  $date = $this->{datestring}; # scalar localtime;
+  $spamreport = $this->{mcpreport};
 
   # Delete everything in brackets after the SA report, if it exists
   $spamreport =~ s/(spamassassin)[^(]*\([^)]*\)/$1/i;
@@ -543,6 +546,8 @@ sub HandleMCPBounceAttachment {
                                         Data        => \@original));
 
   # Stringify the message and send it -- this could be VERY large!
+  # Prune all the dead branches off the tree
+  PruneEntityTree($bounce);
   my $bouncetext = $bounce->stringify;
   #print STDERR "Spam bounce message is this:\n$bouncetext";
   if ($bouncetext) {
@@ -565,6 +570,7 @@ sub HandleMCPNotify {
 
   my($from,$to,$subject,$date,$spamreport,$hostname,$day,$month,$year);
   my($emailmsg, $line, $messagefh, $filename, $localpostmaster, $id);
+  my($postmastername);
 
   $from = $this->{from};
 
@@ -594,15 +600,17 @@ sub HandleMCPNotify {
   # Setup other variables they can use in the message template
   $id = $this->{id};
   $localpostmaster = MailScanner::Config::Value('localpostmaster', $this);
+  $postmastername  = MailScanner::Config::LanguageValue($this, 'mailscanner');
   $hostname = MailScanner::Config::Value('hostname', $this);
   $subject = $this->{subject};
-  $date = scalar localtime;
-  $spamreport = $this->{spamreport};
+  $date = $this->{datestring}; # scalar localtime;
+  $spamreport = $this->{mcpreport};
   # And let them put the date number in there too
-  ($day, $month, $year) = (localtime)[3,4,5];
-  $month++;
-  $year += 1900;
-  my $datenumber = sprintf("%04d%02d%02d", $year, $month, $day);
+  #($day, $month, $year) = (localtime)[3,4,5];
+  #$month++;
+  #$year += 1900;
+  #my $datenumber = sprintf("%04d%02d%02d", $year, $month, $day);
+  my $datenumber = $this->{datenumber};
 
 
   my($to, %tolist);
