@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm,v 1.64.2.28 2004/12/12 16:17:24 jkf Exp $
+#   $Id: MessageBatch.pm,v 1.64.2.31 2005/04/01 16:34:20 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.64.2.28 $, 10;
+$VERSION = substr q$Revision: 1.64.2.31 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -226,7 +226,7 @@ sub SpamChecks {
 
   while(($id, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
-    next unless MailScanner::Config::Value('spamchecks', $message);
+    next unless MailScanner::Config::Value('spamchecks', $message) =~ /1/;
  
     #print STDERR "Spam checks for $id\n";
     $counter += $message->IsSpam();
@@ -371,7 +371,7 @@ sub DeliverUnscanned {
   my($OutQ, @messages, $id, $message);
 
   while(($id,$message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
 
     if (!$message->{scanme}) {
       #print STDERR "Delivering unscanned message $id\n";
@@ -409,6 +409,9 @@ sub Explode {
 
   my($key, $message);
 
+  # jjh 2004-03-12 reap as many as we can.
+  1 until waitpid(-1, &POSIX::WNOHANG) == -1;
+
   #print STDERR "About to explode messages\n";
   umask $global::MS->{work}->{fileumask};
   while(($key, $message) = each %{$this->{messages}}) {
@@ -764,7 +767,7 @@ sub WarnSenders {
     #print STDERR "Looking to warn sender of $id\n";
     next unless $message->{infected};
     #print STDERR "Warning sender of $id who is " .  $message->{from} . "\n";
-    next unless MailScanner::Config::Value('warnsenders', $message);
+    next unless MailScanner::Config::Value('warnsenders', $message) =~ /1/;
     #print STDERR "2Warning sender of $id who is " .  $message->{from} . "\n";
 
     # Count up the number of reasons why we want to warn the sender.
@@ -810,7 +813,7 @@ sub WarnLocalPostmaster {
   while(($id, $message) = each %{$this->{messages}}) {
     next if !$message->{infected};
     next if $message->{deleted} && !$message->{stillwarn};
-    next unless MailScanner::Config::Value('sendnotices', $message);
+    next unless MailScanner::Config::Value('sendnotices', $message) =~ /1/;
     $posties = MailScanner::Config::Value('noticerecipient', $message);
     return if $posties =~ /^\s*$/; # Return if no opsties defined
     @posties = split(" ", $posties);
