@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: PFDiskStore.pm,v 1.1.2.29 2004/12/14 16:29:35 jkf Exp $
+#   $Id: PFDiskStore.pm,v 1.1.2.35 2005/03/31 19:57:31 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use MailScanner::Config;
 use vars qw($VERSION @DeletesPending);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.29 $, 10;
+$VERSION = substr q$Revision: 1.1.2.35 $, 10;
 
 # List of pending delete operations so we can clear up properly when killed
 @DeletesPending = ();
@@ -59,7 +59,7 @@ $VERSION = substr q$Revision: 1.1.2.29 $, 10;
 #use vars qw($VERSION);
 #
 #### The package version, both in 1.23 style *and* usable by MakeMaker:
-#$VERSION = substr q$Revision: 1.1.2.29 $, 10;
+#$VERSION = substr q$Revision: 1.1.2.35 $, 10;
 
 # Attributes are
 #
@@ -94,6 +94,9 @@ sub new {
   } elsif ($MailScanner::SMDiskStore::HashDirDepth == 1) {
     $this->{hdname} =~ /^(.)(.*)$/;
     $this->{hdpath} = "$dir/$1/" . $this->{hdname};
+  } elsif ($MailScanner::SMDiskStore::HashDirDepth == 0) {
+    $this->{hdname} =~ /^(.*)$/;
+    $this->{hdpath} = "$dir/" . $this->{hdname};
   }
   #print STDERR "Created new message object at " . $this->{hdpath} . "\n";
 
@@ -261,6 +264,7 @@ sub WriteHeader {
     if (not defined $pipe or not defined ($pid = fork)) {
       MailScanner::Log::WarnLog("Pipe creation failed in WriteHeader, %s", $!);
     } elsif ($pid) { # Parent
+      $Tf->flush(); # JKF 20050317
       $pipe->reader();
       # Read the pipe a line at a time and write an N record for each line.
       while(<$pipe>) {
@@ -273,14 +277,16 @@ sub WriteHeader {
       # fork and exec, it unfortunately doesn't have a neat hook for us
       # to tell it the pid when we do the fork. Bah.
       $pipe->close();
+      $Tf->flush(); # JKF 20050307
       waitpid $pid, 0;
     } else { # Child
+      $Tf->flush(); # JKF 20050317
       $pipe->writer();
       $entity->print_body($pipe)
         or MailScanner::Log::WarnLog("WriteMIMEBody to %s possibly failed, %s",
                                      $tfile, $!);
       $pipe->close();
-      $Tf->flush();
+      #$Tf->flush(); # JKF 20050307
       exit;
     }
   }
@@ -364,6 +370,16 @@ sub WriteHeader {
       or MailScanner::Log::DieLog("Cannot rename clean %s to %s, %s",
                                   $tfile, $hdoutfile, $!);
     MailScanner::Log::InfoLog("Requeue: %s to %s", $message->{id},$hdoutfile);
+  } elsif ($MailScanner::SMDiskStore::HashDirDepth == 0) {
+    ($hddirbase, $hdoutfile) = 
+      MailScanner::Sendmail::HDOutFileName($tfile);
+    #print STDERR "tfile = $tfile and hdoutfile = $hdoutfile\n";
+    # Update all the datestamps so that Postfix qmgr will see them
+    utime $now, $now, "$tfile";
+    rename "$tfile", "$hddirbase/$hdoutfile"
+      or MailScanner::Log::DieLog("Cannot rename clean %s to %s, %s",
+                                  $tfile, $hdoutfile, $!);
+    MailScanner::Log::InfoLog("Requeue: %s to %s", $message->{id},$hdoutfile);
   }
 }
 
@@ -560,8 +576,8 @@ sub CopyToDir {
     $hdfile = $file;
   } else {
     # We weren't passed a sensible filename, so work one out for ourselves.
-    $hdfile = $this->{id}; #basename($hdpath);
-print STDERR "hdfile = " . $this->{id} . "\n";
+    $hdfile = basename($hdpath); #$hdfile = $this->{id}; #basename($hdpath);
+    #print STDERR "hdfile = $hdfile\n";
   }
   copy($hdpath, "$dir/$hdfile");
   chown $uid, $gid, "$dir/$hdfile" if $changeowner;
