@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Postfix.pm,v 1.1.2.53 2004/12/18 12:25:46 jkf Exp $
+#   $Id: Postfix.pm,v 1.1.2.61 2005/03/31 19:57:31 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.53 $, 10;
+$VERSION = substr q$Revision: 1.1.2.61 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -71,7 +71,7 @@ sub initialise {
   MailScanner::Config::Default('sendmail', '/usr/sbin/sendmail');
   MailScanner::Config::Default('sendmail2',
                                MailScanner::Config::Value('sendmail'));
-  $MailScanner::SMDiskStore::HashDirDepth = 0;
+  $MailScanner::SMDiskStore::HashDirDepth = -1;
   $UnsortedBatchesLeft = 0; # Disable queue-clearing mode
 }
 
@@ -262,6 +262,8 @@ sub new {
     } elsif ($MailScanner::SMDiskStore::HashDirDepth == 1) {
       $file =~ /^(.)/;
       return ($dir,$1,$file);
+    } elsif ($MailScanner::SMDiskStore::HashDirDepth == 0) {
+      return ($dir,$file);
     } else {
       MailScanner::Log::WarnLog("Postfix dir depth has not been set!");
     }
@@ -355,10 +357,10 @@ sub new {
         $recdata =~ s/^\<//;
         $recdata =~ s/\<$//;
         $recdata = lc($recdata);
-        push @{$message->{to}}, $recdata;
+        push @{$message->{to}}, lc($recdata);
         push @{$message->{metadata}}, "$rectype$recdata";
         #JKF 20040322 $message->{originalrecips}{"$recdata"} = 1;
-        $message->{postfixrecips}{"$recdata"} = 1;
+        $message->{postfixrecips}{lc("$recdata")} = 1;
         $TOFound = 1;
         #print STDERR "Pre O Recip $recdata\n";
       } else {
@@ -384,6 +386,12 @@ sub new {
         $message->{subject} = $1;
         next;
       }
+      # JKF Testing code -- Must remove before release
+      #if ($recdata =~ /^X-ClientIP:\s*(\S+)$/i) {
+      #  $message->{clientip} = $1;
+      #  $IPFound = 1;
+      #}
+      # JKF Testing code ends -- Must remove before release
       if (!$IPFound && $recdata =~ /^Received: .+\[(\d+\.\d+\.\d+\.\d+)\]/i) {
         $message->{clientip} = $1;
         $IPFound = 1;
@@ -465,6 +473,9 @@ sub new {
     # we have something there
     $message->{clientip} = '0.0.0.0' unless $IPFound;
 
+    # Decode the ISO encoded Subject line
+    $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
+
     return 1 if $FROMFound && $TOFound; # && $IPFound;
     #MailScanner::Log::WarnLog("Batch: Found invalid queue file for " .
     #                          "message %s", $message->{id});
@@ -606,6 +617,35 @@ sub new {
     }
   }
 
+  sub UniqHeader {
+    my($this, $message, $key) = @_;
+
+    my($pos, $foundat);
+    $pos = 0;
+    $pos++ while ($message->{metadata}[$pos] !~ /^M/);
+    # Now points at the M record
+    $pos++;
+    # Now points at first N record
+    $foundat = -1;
+    while ($pos < @{$message->{metadata}}) {
+      if ($message->{metadata}[$pos] =~ /^N$key/i) {
+        if ($foundat == -1) { # Skip 1st occurrence
+          $foundat = $pos;
+          $pos++;
+          next;
+        }
+        # We have found the start of 1 occurrence of this header
+        splice @{$message->{metadata}}, $pos, 1;
+        # Delete continuation lines
+        while($message->{metadata}[$pos] =~ /^N\s/) {
+          splice @{$message->{metadata}}, $pos, 1;
+        }
+        next;
+      }
+      $pos++;
+    }
+  }
+
   sub ReplaceHeader {
     my($this, $message, $key, $newvalue) = @_;
 
@@ -843,6 +883,7 @@ sub new {
     #print STDERR "In PreDataString\n";
     # Output all the metadata records up until (& including) the M record.
     $linenum = 0;
+    $result  = '';
     foreach (@{$message->{metadata}}) {
       /^(.)(.*)$/;
       ($type, $data) = ($1, $2);
@@ -879,10 +920,11 @@ sub new {
     # Store the length of th estring so far as we need to return it
     $preNlen = length($result);
 
+    my $totallines = scalar(@{$message->{metadata}});
     # Add the headers
-    foreach(; $linenum<@{$message->{metadata}}; $linenum++) {
-      $_ = $message->{metadata}[$linenum];
-      /^(.)(.*)$/;
+    for ($linenum=$linenum; $linenum<$totallines; $linenum++) {
+      #$_ = $message->{metadata}[$linenum];
+      $message->{metadata}[$linenum] =~ /^(.)(.*)$/;
       ($type, $data) = ($1, $2);
       #print STDERR "PreData2 Type $type Data $data\n";
       last if $type eq 'X';
@@ -1072,7 +1114,8 @@ sub FindHashDirDepth {
 
     # Didn't find anything at all, so sleep waiting for a file or a dir
     # to appear in the queue.
-    MailScanner::Log::WarnLog("Messages found but no hashed queue directories. Please enable hashed queues for incoming and deferred with a depth of 1 or 2. See the Postfix documentation for hash_queue_names and hash_queue_depth")
+    # Can now be 0, 1 or 2: MailScanner::Log::WarnLog("Messages found but no hashed queue directories. Please enable hashed queues for incoming and deferred with a depth of 1 or 2. See the Postfix documentation for hash_queue_names and hash_queue_depth")
+    return 0
       if $filecount>0;
     sleep($delay);
   }
@@ -1152,7 +1195,7 @@ sub FindHashDirDepth {
         # If we haven't found the hash directory depth yet, then work it out
         #MailScanner::Log::WarnLog("JKF: About to work out hash dir depth");
         $MailScanner::SMDiskStore::HashDirDepth = FindHashDirDepth($mta)
-          unless $MailScanner::SMDiskStore::HashDirDepth;
+          unless $MailScanner::SMDiskStore::HashDirDepth >= 0;
         #MailScanner::Log::WarnLog("JKF: Hash dir depth is %d",
         #  $MailScanner::SMDiskStore::HashDirDepth);
 
@@ -1165,6 +1208,22 @@ sub FindHashDirDepth {
         # files in the the child directories.
         while(defined($file = $queuedir->read())) {
           next if $file eq '.' || $file eq '..';
+          if ($MailScanner::SMDiskStore::HashDirDepth==0) {
+            next unless $file =~ /$mta->{HDFileRegexp}/;
+            push @SortedFiles, "$queuedirname/$file";
+            if ($UnsortedBatchesLeft<=0) {
+              # Running normally
+              ($nlinks, $tmpdate) = (stat($file))[3,9]; # 9 = mtime
+              next if -z _;
+              next unless -f _;
+              next unless -R _;
+              next if $nlinks>1; # Catch files being moved into "deferred"
+              $ModDate{"$queuedirname/$file"} = $tmpdate;
+            }
+            $MsgsInQueue++;
+            #print STDERR "Stored depth 0 message file $file\n";
+            next;
+          }
           next unless -d $file;
           $queue1dir->open($file) or next;
           while(defined($file1 = $queue1dir->read())) {
@@ -1257,6 +1316,9 @@ sub FindHashDirDepth {
         ($queuedirname, $h1, $file) = ($1,$2,$3)
           if $MailScanner::SMDiskStore::HashDirDepth == 1 &&
              $file =~ /^(.*)\/(.)\/([^\/]+)$/;
+        ($queuedirname, $file) = ($1,$2)
+          if $MailScanner::SMDiskStore::HashDirDepth == 0 &&
+             $file =~ /^(.*)\/([^\/]+)$/;
         next unless $file =~ /$mta->{HDFileRegexp}/;
         # Put the real message id in $idorig and the unique name in $id
         # JKF Add a dot followed by a random number to try to get a unique
@@ -1280,7 +1342,8 @@ sub FindHashDirDepth {
         #print STDERR "Added $id to batch\n";
         $batchempty = 0;
 
-        if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/) {
+        if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+            MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
           $newmessage->NeedsScanning(1);
           $DirtyMsgs++;
           $DirtyBytes += $newmessage->{size};
