@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SA.pm,v 1.35.2.47 2004/12/14 16:29:36 jkf Exp $
+#   $Id: SA.pm,v 1.35.2.49 2005/03/12 20:15:19 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use POSIX qw(:signal_h); # For Solaris 9 SIG bug workaround
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.35.2.47 $, 10;
+$VERSION = substr q$Revision: 1.35.2.49 $, 10;
 
 # Attributes are
 #
@@ -126,10 +126,19 @@ sub initialise {
     # Rebuild the Bayes database if it is due
     $MailScanner::SA::BayesRebuildLock = MailScanner::Config::Value(
                           'lockfiledir') . '/MS.bayes.rebuild.lock';
+    $MailScanner::SA::BayesRebuildStartLock =
+      MailScanner::Config::Value('lockfiledir') . '/MS.bayes.starting.lock';
     $MailScanner::SA::WaitForRebuild = MailScanner::Config::Value('bayeswait');
     $MailScanner::SA::DoingBayesRebuilds = MailScanner::Config::Value('bayesrebuild');
     if ($RebuildBayes) {
-      MailScanner::Log::InfoLog('SpamAssassin Bayes database rebuild preparing');
+      #MailScanner::Log::InfoLog('SpamAssassin Bayes database rebuild preparing');
+      # Tell the other children that we are trying to start a rebuild
+      my $RebuildStartH = new FileHandle;
+      unless ($RebuildStartH->open("+>$MailScanner::SA::BayesRebuildStartLock")) {
+        MailScanner::Log::WarnLog("Bayes rebuild process could not write to " .
+                                  "%s to signal starting",
+                                  $MailScanner::SA::BayesRebuildStartLock);
+      }
 
       # Get an exclusive lock on the bayes rebuild lock file
       my $RebuildLockH = new FileHandle;
@@ -162,7 +171,10 @@ sub initialise {
         $RebuildLockH->close();
         MailScanner::Log::InfoLog('SpamAssassin Bayes database rebuild completed');
       }
-      MailScanner::Log::InfoLog('Rebuilding SpamAssassin Bayes database');
+
+      # Now the rebuild has properly finished, we let the other children back
+      unlink $MailScanner::SA::BayesRebuildStartLock;
+      $RebuildStartH->close();
     }
 
     if (MailScanner::Config::Value('spamassassinautowhitelist')) {
@@ -265,6 +277,31 @@ sub Checks {
   my $BayesIsLocked = 0;
   my($RebuildLockH, $Lockopen);
   if ($MailScanner::SA::DoingBayesRebuilds) {
+    # If the lock file exists at all, do not try to get a lock on it.
+    # Shared locks are handed out even when someone else is trying to
+    # get an exclusive lock, so long as at least 1 other shared lock
+    # already exists.
+    if (-e $MailScanner::SA::BayesRebuildStartLock) {
+      # Do we wait for Bayes rebuild to occur?
+      if ($MailScanner::SA::WaitForRebuild) {
+        # Wait quietly for the file to disappear
+        # This must not take more than 1 hour or we are in trouble!
+        #MailScanner::Log::WarnLog("Waiting for rebuild start request to disappear");
+        my $waiter = 0;
+        for ($waiter = 0; $waiter<3600 &&
+             -e $MailScanner::SA::BayesRebuildStartLock; $waiter+=10) {
+          sleep 10;
+          #MailScanner::Log::WarnLog("Waiting for start request to disappear");
+        }
+        # Did it take too long?
+        unlink $MailScanner::SA::BayesRebuildStartLock if $waiter>=3590;
+        #MailScanner::Log::WarnLog("Start request has disappeared");
+      } else {
+        # Return saying we are skipping SpamAssassin this time
+        return (0,0, 'SpamAssassin rebuilding', 0);
+      }
+    }
+
     $Lockopen = 0;
     $RebuildLockH = new FileHandle;
 
@@ -288,15 +325,12 @@ sub Checks {
 
     if ($Lockopen) {
       #MailScanner::Log::InfoLog("Bayes lock is open");
-      #flock($RebuildLockH, $LOCK_SH) or MailScanner::Log::WarnLog("Bayes2 unlock said %s", $!);
-      #flock($RebuildLockH, ($LOCK_SH | $LOCK_NB)) and MailScanner::Log::WarnLog("************ Bayes2 lock said %s", $!);
       if ($MailScanner::SA::WaitForRebuild) {
         # Do a normal lock and wait for it
         flock($RebuildLockH, $LOCK_SH) or
           MailScanner::Log::WarnLog("At start of SA checks could not get " .
             "shared lock on %s, %s", $MailScanner::SA::BayesRebuildLock, $!);
         $BayesIsLocked = 1;
-        #MailScanner::Log::InfoLog("Waiting for Bayes database to rebuild");
       } else {
       #MailScanner::Log::InfoLog("Bayes lock2 is %s", $RebuildLockH);
         if (flock($RebuildLockH, ($LOCK_SH | $LOCK_NB))) {
@@ -305,7 +339,7 @@ sub Checks {
         } else {
       #MailScanner::Log::InfoLog("Skipping Bayes due to %s", $!);
           $RebuildLockH->close();
-          MailScanner::Log::InfoLog("Skipping SpamAssassin while waiting for Bayes database to rebuild");
+          #MailScanner::Log::InfoLog("Skipping SpamAssassin while waiting for Bayes database to rebuild");
           return (0,0, 'SpamAssassin rebuilding', 0);
         }
       }
