@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Sendmail.pm,v 1.56.2.29 2004/12/18 12:25:47 jkf Exp $
+#   $Id: Sendmail.pm,v 1.56.2.34 2005/04/06 17:16:05 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.56.2.29 $, 10;
+$VERSION = substr q$Revision: 1.56.2.34 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -48,7 +48,7 @@ $VERSION = substr q$Revision: 1.56.2.29 $, 10;
 my $SendmailOptions = "-t -oi -oem -F MailScanner -f";
 my $UnsortedBatchesLeft;
 # This is true if the queue dir contains qf,df,xf sub-dirs.
-my %IsNestedQueue = ();
+my %IsNestedQueue;
 
 # Attributes are
 #
@@ -68,7 +68,9 @@ sub initialise {
   MailScanner::Config::Default('sendmail2',
                                MailScanner::Config::Value('sendmail'));
   $UnsortedBatchesLeft = 0; # Disable queue-clearing mode
-  %MailScanner::Sendmail::IsNestedQueue = ();
+  #%MailScanner::Sendmail::IsNestedQueue = ();
+  #print STDERR "Clearing IsNestedQueue\n";
+  %IsNestedQueue = ();
 }
 
 # Constructor.
@@ -236,6 +238,7 @@ my($sed) = "/bin/sed";
 
   sub DFileName {
     my($this, $id, $dir) = @_;
+    #print STDERR "df IsNested $dir = " . $IsNestedQueue{$dir} . "\n";
     return "df/df$id" if $IsNestedQueue{$dir};
     return "df$id";
   }
@@ -243,6 +246,7 @@ my($sed) = "/bin/sed";
   # No change for V4
   sub HFileName {
     my($this, $id, $dir) = @_;
+    #print STDERR "qf IsNested $dir = " . $IsNestedQueue{$dir} . "\n";
     return "qf/qf$id" if $IsNestedQueue{$dir};
     return "qf$id";
   }
@@ -250,6 +254,7 @@ my($sed) = "/bin/sed";
   # No change for V4
   sub TFileName {
     my($this, $id, $dir) = @_;
+    #print STDERR "tf IsNested $dir = " . $IsNestedQueue{$dir} . "\n";
     return "tf/tf$id" if $IsNestedQueue{$dir};
     return "tf$id";
   }
@@ -312,7 +317,8 @@ my($sed) = "/bin/sed";
       if ($Line =~ /^\$_/) {
         $ip = $Line;
         #chomp $ip;
-        if ($ip =~ /\[([\d.:abcdef]+)\]/) {
+        # Linux adds "IPv6:" on the front of the IPv6 address, so remove it
+        if ($ip =~ /\[(?:IPv6:)?([\d.:abcdef]+)\]/) {
           $message->{clientip} = $1;
         } else {
           # It is a locally-created message and doesn't have an smtp client ip
@@ -349,6 +355,9 @@ my($sed) = "/bin/sed";
     # So provide an IP if we haven't found one.
     $message->{clientip} = '0.0.0.0' unless $IPFound;
 
+    # Decode the ISO encoded Subject line
+    $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
+
     # Every qf file should at least define the sender, 1 recipient and the
     # IP address. Everything else is optional, and is preserved as
     # MailScanner may not understand all the types of line.
@@ -411,6 +420,27 @@ my($sed) = "/bin/sed";
     }
   }
 
+  sub UniqHeader {
+    my($this, $message, $key) = @_;
+
+    my $linenum;
+    my $foundat = -1;
+    for ($linenum=0; $linenum<@{$message->{metadata}}; $linenum++) {
+      next unless $message->{metadata}[$linenum] =~ /^H(\?[^?]*\?)?$key/i;
+
+      # Have found the header line, skip it if we haven't seen it before
+      ($foundat = $linenum), next if $foundat == -1;
+
+      # Have found the right line
+      splice(@{$message->{metadata}}, $linenum, 1);
+      # Now delete the continuation lines
+      while($message->{metadata}[$linenum] =~ /^\s/) {
+        splice(@{$message->{metadata}}, $linenum, 1);
+      }
+      $linenum--; # Allow for 2 neighbouring instances of $key
+    }
+  }
+
   sub ReplaceHeader {
     my($this, $message, $key, $newvalue) = @_;
 
@@ -797,9 +827,10 @@ my($sed) = "/bin/sed";
       while( defined($queuedirname=splice(@aux_queuedirnames,
             ($UnsortedBatchesLeft<=0 ? 0 :int(rand(@aux_queuedirnames))),1))) {
         #print STDERR "Scanning dir $queuedirname\n";
+        #print STDERR "IsNestedQueue = " . join(',',%IsNestedQueue) . "\n";
         if ($IsNestedQueue{$queuedirname}) {
           # Queue is nested, so $queuedirname ends with /qf
-          print STDERR "$queuedirname is nested\n";
+          #print STDERR "$queuedirname is nested\n";
           $queuedirname .= '/qf';
           unless (chdir $queuedirname) {
             MailScanner::Log::WarnLog("Cannot cd to dir %s to read messages, %s",
@@ -811,14 +842,14 @@ my($sed) = "/bin/sed";
                                         "reading message batch, %s",
                                         $queuedirname, $!);
           $mta = $global::MS->{mta};
-          print STDERR "Searching " . $queuedirname . " for messages\n";
+          #print STDERR "Searching " . $queuedirname . " for messages\n";
 
           # Read in modification dates of the qf files & use them in date order
           while(defined($file = $queuedir->read())) {
-            print STDERR "Found $file\n";
+            #print STDERR "Found $file\n";
             # Optimised by binning the 50% that aren't H files first
             next unless $file =~ /$mta->{HFileRegexp}/;
-            print STDERR "Found message file $file\n";
+            #print STDERR "Found message file $file\n";
             $MsgsInQueue++; # Count the size of the queue
             push @SortedFiles, "$queuedirname/$file";
             if ($UnsortedBatchesLeft<=0) {
@@ -827,7 +858,7 @@ my($sed) = "/bin/sed";
               next unless -f _;
               next if -z _; # Skip 0-length qf files
               $ModDate{"$queuedirname/$file"} = $tmpdate; # Push msg into list
-              print STDERR "Stored message file $file\n";
+              #print STDERR "Stored message file $file\n";
             }
           }
           $queuedir->close();
@@ -917,7 +948,8 @@ my($sed) = "/bin/sed";
         $batch->{messages}{"$id"} = $newmessage;
         $batchempty = 0;
 
-        if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/) {
+        if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+            MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
           $newmessage->NeedsScanning(1);
           $DirtyMsgs++;
           $DirtyBytes += $newmessage->{size};
@@ -1044,7 +1076,8 @@ sub CheckQueueIsFlat {
   $dirhandle->close();
 
   # Remember the dir was nested if necessary
-  $IsNestedQueue{$dir} = 1 if $FoundQf && $FoundDf;
+  $IsNestedQueue{$dir} = ($FoundQf && $FoundDf)?1:0;
+  #print STDERR "Set IsNestedQueue for $dir so now " . join(',',%IsNestedQueue) . "\n";
   MailScanner::Log::InfoLog("Queue directory %s is nested", $dir)
     if $FoundQf && $FoundDf;
 
