@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepContent.pm,v 1.17.2.19 2004/11/04 14:34:30 jkf Exp $
+#   $Id: SweepContent.pm,v 1.17.2.23 2005/02/23 13:30:24 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.17.2.19 $, 10;
+$VERSION = substr q$Revision: 1.17.2.23 $, 10;
 
 # Attributes are
 #
@@ -438,22 +438,20 @@ sub SearchHTMLBody {
      $convertwebbugs,
      $stripdangerous) = @_;
 
-  # We must say we have tags to convert if we are detecting phishing so
-  # that the message is rebuilt properly before delivery.
-  $message->{tagstoconvert} .= 'phishing '
-    if MailScanner::Config::Value('findphishing', $message);
-
   my($fh, $counter, $silentviruses);
   $counter = 0;
   $fh = new FileHandle;
   if ($fh->open("$filename")) {
 
+    my $loggingtags = MailScanner::Config::Value('loghtmltags', $message);
+
     # Search the file
     my $inobject    = 0;
     my $iframefound = 0;
     my $formfound   = 0;
     my $scriptfound = 0;
     my $webbugfound = 0;
+    my $phishingfound = 0;
     my $codebasefound = 0;
     my $attach = $filename;
     $attach = $1 if $filename =~ /([^\/]+)$/; # Strip off the path
@@ -473,6 +471,9 @@ sub SearchHTMLBody {
       # Find the img tag start
       $webbugfound = 1 if /\<img/i;
 
+      # Find the link tag start
+      $phishingfound = 1 if /\<a/i;
+
       # Find the object tag start
       $inobject = 1 if /\<object/i;
 
@@ -489,11 +490,18 @@ sub SearchHTMLBody {
     $silentviruses = ' ' .  MailScanner::Config::Value('silentviruses',
                      $message) . ' ';
 
+    if ($phishingfound && MailScanner::Config::Value('findphishing', $message)){
+      # Log the <A>
+      MailScanner::Log::InfoLog("<A> tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
+      # Mark the message
+      $message->{tagstoconvert} .= 'phishing ';
+      #$message->{bodymodified}   = 1;
+    }
     if ($iframefound) {
       # Log the <IFrame>
-      MailScanner::Log::InfoLog("HTML IFrame tag found in message %s from %s",
-        $id, $message->{from})
-        if MailScanner::Config::Value('logiframetags', $message);
+      MailScanner::Log::InfoLog("HTML-IFrame tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       # Mark the message
       if ($allowiframes) {
         if ($stripdangerous) {
@@ -514,10 +522,9 @@ sub SearchHTMLBody {
       }
     }
     if ($formfound) {
-      ## Log the <IFrame>
-      #MailScanner::Log::InfoLog("HTML Form tag found in message %s from %s",
-      #  $id, $message->{from})
-      #  if MailScanner::Config::Value('logformtags', $message);
+      ## Log the <Form>
+      MailScanner::Log::InfoLog("HTML-Form tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       # Mark the message
       if ($allowforms) {
         if ($stripdangerous) {
@@ -539,9 +546,8 @@ sub SearchHTMLBody {
     }
     if ($scriptfound) {
       ## Log the <script>
-      #MailScanner::Log::InfoLog("HTML Form tag found in message %s from %s",
-      #  $id, $message->{from})
-      #  if MailScanner::Config::Value('logformtags', $message);
+      MailScanner::Log::InfoLog("HTML-Script tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       # Mark the message
       if ($allowscripts) {
         if ($stripdangerous) {
@@ -578,7 +584,9 @@ sub SearchHTMLBody {
       } elsif ($convertwebbugs) {
         #print STDERR "Web Bug converted\n";
         $message->{tagstoconvert} .= 'webbug ';
-        $message->{bodymodified}   = 1;
+        # Only mark it for rebuilding if we actually found a webbug
+        # as we shouldn't rebuild if it was an innocent image
+        #$message->{bodymodified}   = 1;
         #print STDERR "Going to disarm web bugs\n";
       } else {
         #print STDERR "Web Bug ignored\n";
@@ -596,6 +604,8 @@ sub SearchHTMLBody {
       }
     }
     if ($codebasefound) {
+      MailScanner::Log::InfoLog("HTML-Object tag found in message %s from %s",
+        $id, $message->{from}) if $loggingtags;
       if ($allowobjects) {
         if ($stripdangerous) {
           $message->{needsstripping} = 1;
