@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepViruses.pm,v 1.49.2.92 2004/12/29 17:27:34 jkf Exp $
+#   $Id: SweepViruses.pm,v 1.49.2.102 2005/04/27 07:30:58 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION $ScannerPID);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.49.2.92 $, 10;
+$VERSION = substr q$Revision: 1.49.2.102 $, 10;
 
 # Locking definitions for flock() which is used to lock the Lock file
 my($LOCK_SH) = 1;
@@ -331,7 +331,7 @@ my %Scanners = (
     InitParser          => \&InitDrwebParser,
     ProcessOutput       => \&ProcessDrwebOutput,
     SupportScanning     => $S_SUPPORTED,
-    SupportDisinfect    => $S_ALPHA,
+    SupportDisinfect    => $S_SUPPORTED,
   },
   "norman"   => {
     Name		=> 'Norman',
@@ -369,9 +369,11 @@ my %Scanners = (
   "vexira"   => {
     Name                => 'Vexira',
     Lock                => 'VexiraBusy.lock',
-    CommonOptions       => '--allfiles -s -z -noboot -nombr -r1 -rs -lang=EN --alltypes',
-    DisinfectOptions    => '-e',
-    ScanOptions         => '',
+    #CommonOptions       => '--allfiles -s -z -noboot -nombr -r1 -rs -lang=EN --alltypes',
+    #DisinfectOptions    => '-e',
+    CommonOptions       => '-qq --scanning=full',
+    DisinfectOptions    => '--remove-macro --action=kill',
+    ScanOptions         => '--action=skip',
     InitParser          => \&InitVexiraParser,
     ProcessOutput       => \&ProcessVexiraOutput,
     SupportScanning     => $S_SUPPORTED,
@@ -385,8 +387,8 @@ my %Scanners = (
     ScanOptions		=> '-mode scan',
     InitParser		=> \&InitSymScanEngineParser,
     ProcessOutput	=> \&ProcessSymScanEngineOutput,
-    SupportScanning	=> $S_BETA,
-    SupportDisinfect	=> $S_BETA,
+    SupportScanning	=> $S_SUPPORTED,
+    SupportDisinfect	=> $S_SUPPORTED,
   },
   "none"		=> {
     Name		=> 'None',
@@ -419,6 +421,15 @@ sub initialise {
     #print STDERR "ClamAV Module in use\n";
     InitialiseClam();
   }
+  # Set the Unrar command path for the ClamAV code
+  if (grep /^clamav$/, @scanners) {
+    my $rarcmd = MailScanner::Config::Value('unrarcommand');
+    if ($rarcmd && -x $rarcmd) {
+      $Scanners{clamav}->{CommonOptions} .= " --unrar=$rarcmd";
+      MailScanner::Log::InfoLog("ClamAV scanner using unrar command %s",
+                                $rarcmd);
+    }
+  }
 }
 
 sub InitialiseClam {
@@ -436,9 +447,12 @@ sub InitialiseClam {
     or MailScanner::Log::DieLog("ClamAV Module ERROR:: Could not load " .
        "databases from %s", Mail::ClamAV::retdbdir());
   $Clam->buildtrie;
-  $Clam->maxreclevel(5);             # Impose limits
-  $Clam->maxfiles(500);              # Impose limits
-  $Clam->maxfilesize(500*1024*1024); # Impose limits
+  # Impose limits
+  $Clam->maxreclevel(MailScanner::Config::Value('clamavmaxreclevel'));
+  $Clam->maxfiles   (MailScanner::Config::Value('clamavmaxfiles'));
+  $Clam->maxfilesize(MailScanner::Config::Value('clamavmaxfilesize'));
+  $Clam->maxratio   (MailScanner::Config::Value('clamavmaxratio'));
+
 
   # Build the hash of the size of all the watch files
   my(@watchglobs, $glob, @filelist, $file, $filecount);
@@ -454,8 +468,8 @@ sub InitialiseClam {
   MailScanner::Log::DieLog("None of the files matched by the \"Monitors " .
     "For ClamAV Updates\" patterns exist!") unless $filecount>0;
 
-  MailScanner::Log::WarnLog("\"Allow Password-Protected Archives\" should be set to just yes or no when using clamavmodule virus scanner")
-    unless MailScanner::Config::IsSimpleValue('allowpasszips');
+  #MailScanner::Log::WarnLog("\"Allow Password-Protected Archives\" should be set to just yes or no when using clamavmodule virus scanner")
+  #  unless MailScanner::Config::IsSimpleValue('allowpasszips');
 }
 
 
@@ -638,7 +652,7 @@ sub ScanBatch {
 
   #print STDERR (($ScanType =~ /dis/i)?"Disinfecting":"Scanning") . " using ".
   #             "commercial virus scanners\n";
-  $success = TryCommercial('.', $BaseDir, \%Reports, \%Types,
+  $success = TryCommercial($batch, '.', $BaseDir, \%Reports, \%Types,
                            \$NumInfections, $ScanType);
   #print STDERR "Found $NumInfections infections\n";
   unless ($success) {
@@ -652,6 +666,7 @@ sub ScanBatch {
     while(defined($id = $BaseDirH->read())) {
       next unless -d "$id";   # Only check directories
       next if $id =~ /^\.+$/; # Don't check myself or my parent
+      next unless MailScanner::Config::Value('virusscan',$batch->{messages}{id}) =~ /1/;
       # The "./" is important as it gets the path right for parser code
       $success = TryCommercial("./$id", $BaseDir, \%Reports,
                                \%Types, \$NumInfections, $ScanType);
@@ -696,6 +711,8 @@ sub MergeReports {
     #print STDERR "Report merging for \"$id\" and \"$reports\"\n";
     next unless $id && $reports;
     my $message = $batch->{messages}{"$id"};
+    # Skip this message if we didn't actually want it to be scanned.
+    next unless MailScanner::Config::Value('virusscan', $message) =~ /1/;
     #print STDERR "Message is $message\n";
     $message->{virusinfected} = 1;
 
@@ -742,7 +759,7 @@ sub MergeReports {
 #               a ref to the infections counter.
 # $ScanType can be one of "scan", "rescan", "disinfect".
 sub TryCommercial {
-  my($dir, $BaseDir, $Reports, $Types, $rCounter, $ScanType) = @_;
+  my($batch, $dir, $BaseDir, $Reports, $Types, $rCounter, $ScanType) = @_;
 
   my($scanner, $scannerlist, @scanners, $disinfect, $result, $counter);
   my($logtitle);
@@ -767,7 +784,7 @@ sub TryCommercial {
   foreach $scanner (@scanners) {
     $result = TryOneCommercial($scanner,
                                MailScanner::Config::ScannerCmds($scanner),
-                               $dir, $BaseDir, $Reports, $Types,
+                               $batch, $dir, $BaseDir, $Reports, $Types,
                                $rCounter, $disinfect);
     $counter += $result;
     MailScanner::Log::InfoLog("%s: %s found %d infections", $logtitle,
@@ -781,8 +798,8 @@ sub TryCommercial {
 
 # Try one of the commercial virus scanners
 sub TryOneCommercial {
-  my($scanner, $sweepcommandAndPath, $subdir, $BaseDir, $Reports, $Types,
-     $rCounter, $disinfect) = @_;
+  my($scanner, $sweepcommandAndPath, $batch, $subdir, $BaseDir,
+     $Reports, $Types, $rCounter, $disinfect) = @_;
 
   my($sweepcommand, $instdir);
   my($rScanner, $VirusLock, $voptions, $Name);
@@ -892,7 +909,7 @@ sub TryOneCommercial {
         SophosSAVI($subdir, $disinfect);
         exit;
       } elsif ($scanner eq 'clamavmodule') {
-        ClamAVModule($subdir, $disinfect);
+        ClamAVModule($subdir, $disinfect, $batch);
         exit;
       } else {
         exec "$sweepcommand $instdir $voptions $subdir";
@@ -947,7 +964,7 @@ sub TryOneCommercial {
 # a directory. Outputs in a very simple format that ProcessClamAVModOutput()
 # expects. 3 output fields separated by ":: ".
 sub ClamAVModule {
-  my($dirname, $disinfect) = @_;
+  my($dirname, $disinfect, $messagebatch) = @_;
 
   my($dir, $child, $childname, $filename, $results, $virus);
 
@@ -960,6 +977,14 @@ sub ClamAVModule {
   #  ;
   #}
 
+  # Do we have an unrar on the path?
+  my $unrar = MailScanner::Config::Value('unrarcommand');
+  MailScanner::Log::WarnLog("Unrar command %s does not exist or is not " .
+    "executable, please either install it or remove the setting from " .
+    "MailScanner.conf", $unrar)
+    unless $unrar eq "" || -x $unrar;
+  my $haverar = 1 if $unrar && -x $unrar;
+
   $| = 1;
   $dir   = new DirHandle;
   $child = new DirHandle;
@@ -985,25 +1010,20 @@ sub ClamAVModule {
     # then have clamav check for password protected archives but it will
     # be reported as a virus (at least it will block passworded rar files)
 
-    # Do we have an unrar on the path?
-    #my $unrar = `which unrar`;
-    #chomp($unrar);
-    #my $haverar = 0;
-    ##MailScanner::Log::InfoLog("*********Check Rar Results : %s\n",$unrar);
-    #$haverar = 1 if $unrar !~ /no unrar in/i && $unrar ne "";
-
     while($filename = $child->read()) {
       next unless -f "$dirname/$childname/$filename"; # Only check files
-      # JKF Patch suggested by Rick Cooper to support unrar and OLE2 files
-      #$results = $Clam->scan("$dirname/$childname/$filename",
-      #                       Mail::ClamAV::CL_SCAN_ARCHIVE());
-      if (MailScanner::Config::Value('allowpasszips')) { # || $haverar) {
+      if (MailScanner::Config::Value('allowpasszips',
+                $messagebatch->{messages}{$childname})) { # || $haverar) {
         $results = $Clam->scan("$dirname/$childname/$filename",
+                               Mail::ClamAV::CL_SCAN_STDOPT() |
                                Mail::ClamAV::CL_SCAN_ARCHIVE() |
+                               Mail::ClamAV::CL_SCAN_PE() |
                                Mail::ClamAV::CL_SCAN_OLE2());
       } else {
         $results = $Clam->scan("$dirname/$childname/$filename",
+                               Mail::ClamAV::CL_SCAN_STDOPT() |
                                Mail::ClamAV::CL_SCAN_ARCHIVE() |
+                               Mail::ClamAV::CL_SCAN_PE() |
                                Mail::ClamAV::CL_SCAN_BLOCKENCRYPTED() |
                                Mail::ClamAV::CL_SCAN_OLE2());
       }
@@ -1226,8 +1246,9 @@ sub InitAvgParser {
 }
 
 # Initialise any state variables the Vexira output parser uses
+my($VexiraPathname);
 sub InitVexiraParser {
-  ;
+  $VexiraPathname = '';
 }
 
 # Initialise any state variables the ScanEngine output parser uses
@@ -1272,10 +1293,10 @@ sub ProcessClamAVModOutput {
   #print STDERR "Output is \"$logout\"\n";
   ($keyword, $virusname, $filename) = split(/:: /, $line, 3);
 
-  if ($keyword =~ /^error/i) {
+  if ($keyword =~ /^error/i && $logout !~ /rar module failure/i) {
     MailScanner::Log::InfoLog("ClamAVModule::%s", $logout);
     return 1;
-  } elsif ($keyword =~ /^info/i) {
+  } elsif ($keyword =~ /^info/i || $logout =~ /rar module failure/i) {
     return 0;
   } elsif ($keyword =~ /^clean/i) {
     return 0;
@@ -1879,6 +1900,10 @@ sub ProcessFProtOutput {
     $line =~ s/(is|could be) a (joke|destructive) program.*$/Infection: /;
   } elsif ($line =~ /(is|could be) infected with an unknown virus/) {
     $line =~ s/(is|could be) infected with an unknown virus.*$/Infection: /;
+  } elsif ($line =~ /(is|could be) a suspicious file/) {
+    $line =~ s/(is|could be) a suspicious file.*$/Infection: /;
+  } elsif ($line =~ /(is|could be) an archive bomb/) {
+    $line =~ s/(is|could be) an archive bomb.*$/Infection: /;
   } elsif ($line =~ /contains.*\(non-working\)/) {
     $line =~ s/contains /Infection: /;
   #} elsif ($line =~ /[Nn]ot scanned \(encrypted\)/) {
@@ -2305,7 +2330,7 @@ sub ProcessClamAVOutput {
     $clamav_archive = $1;
     return 0;
   }
-  return 0 if /Empty file.$/;
+  return 0 if /Empty file\.?$/;
   # Normally means you just havn't asked for it
   if (/: (\S+ module failure\.)/)
   {
@@ -2643,46 +2668,81 @@ sub ProcessAvgOutput {
 sub ProcessVexiraOutput {
   my($line, $infections, $types, $BaseDir, $Name) = @_;
   chomp $line;
-  # Sample output:
-  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.zip --> eicar.com <<< Contains code of the Eicar-Test-Signatur virus
-  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.com <<< Contains code of the Eicar-Test-Signatur virus
-  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.doc <<< Contains code of the Eicar-Test-Signatur virus
 
-  print STDERR "Line: $line\n";
-  return 0 unless $line =~ /^ALERT: \[([^\]]+)\] /;
+  # Interesting output is either a filename starting with ./ or
+  # a virus report starting with whitespace
+  return 0 unless $line =~ /^\.\/|^\s+/;
 
-  my $virus = $1;
-  print STDERR "Line = $line\n";
-  print STDERR "virus = \"$virus\"\n";
-  my $logout = $line;
-  $logout =~ s/\s{20,}/ /g;
-  MailScanner::Log::InfoLog("%s", $logout);
+  # Is it a filename?
+  if ($line =~ /^\.\//) {
+    $VexiraPathname = $line;
+    return 0;
+  }
 
-  # Change all the spaces into / for the split coming up
-  # Also the second variant prepends the archive name to the
-  # infected filename with a:\ so we need to change that to
-  # something else. I chose another / so it would end up in the
-  # @rest wich is also why I changed the \s+ to /
-  # then Remove path elements before /./ leaving just id/part/rest
+  # Dig the message id and attachment filename out of the VexiraPathname
+  my($dot, $id, $part, @rest, $virusname);
+  ($dot, $id, $part, @rest) = split(/\//, $VexiraPathname);
+
+  $line =~ s/^\s+//g;
+  $line =~ s/\s+$//g;
+
+  # virus found: EICAR_test_file ... (NOT killable) skipped
+  #print STDERR "Line is \"$line\"\n";
+  $virusname = $1 if $line =~ /found:\s+(\S+)\s+\.\.\./i;
+  #print STDERR "Virusname is \"$virusname\"\n";
+  MailScanner::Log::InfoLog("Vexira: found %s in %s (%s)", $virusname,
+                            $id, $part);
+  #print STDERR "Id is \"$id\"\nPart is \"$part\"\n";
 
-  #$line =~ s/^ALERT: //g; Patch provided by Alex Kerkhove
-  $line =~ s/^ALERT: //;
-  $line =~ s/\[.+\] *//g;
-  $line =~ s/\.\///;
-  #$line =~ s/^([^\/]+)\///g; Patch provided by Alex Kerkhove
-  $line =~ s/^([^\/]+)\///;
-  my $id = $1;
-  $line =~ /^(.+) <<< (.+)$/;
-  my $part = $1;
-  $part =~ s/ -->.*$//g;
-  print STDERR "id:$id:part = $part\n";
-  print STDERR "$Name : Found virus $virus in file $part ID:$id\n";
   $infections->{$id}{$part} .= $Name . ': ' if $Name;
-  $infections->{$id}{$part} .= "Found $virus in file $part\n";
+  $infections->{$id}{$part} .= "Found virus $virusname in file $part\n";
   $types->{$id}{$part}      .= "v"; # so we know what to tell sender
   return 1;
 }
 
+#sub ProcessVexiraOutput {
+#  my($line, $infections, $types, $BaseDir, $Name) = @_;
+#  chomp $line;
+#  # Sample output:
+#  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.zip --> eicar.com <<< Contains code of the Eicar-Test-Signatur virus
+#  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.com <<< Contains code of the Eicar-Test-Signatur virus
+#  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.doc <<< Contains code of the Eicar-Test-Signatur virus
+#
+#  print STDERR "Line: $line\n";
+#  return 0 unless $line =~ /^ALERT: \[([^\]]+)\] /;
+#
+#  my $virus = $1;
+#  print STDERR "Line = $line\n";
+#  print STDERR "virus = \"$virus\"\n";
+#  my $logout = $line;
+#  $logout =~ s/\s{20,}/ /g;
+#  MailScanner::Log::InfoLog("%s", $logout);
+#
+#  # Change all the spaces into / for the split coming up
+#  # Also the second variant prepends the archive name to the
+#  # infected filename with a:\ so we need to change that to
+#  # something else. I chose another / so it would end up in the
+#  # @rest wich is also why I changed the \s+ to /
+#  # then Remove path elements before /./ leaving just id/part/rest
+#
+#  #$line =~ s/^ALERT: //g; Patch provided by Alex Kerkhove
+#  $line =~ s/^ALERT: //;
+#  $line =~ s/\[.+\] *//g;
+#  $line =~ s/\.\///;
+#  #$line =~ s/^([^\/]+)\///g; Patch provided by Alex Kerkhove
+#  $line =~ s/^([^\/]+)\///;
+#  my $id = $1;
+#  $line =~ /^(.+) <<< (.+)$/;
+#  my $part = $1;
+#  $part =~ s/ -->.*$//g;
+#  print STDERR "id:$id:part = $part\n";
+#  print STDERR "$Name : Found virus $virus in file $part ID:$id\n";
+#  $infections->{$id}{$part} .= $Name . ': ' if $Name;
+#  $infections->{$id}{$part} .= "Found $virus in file $part\n";
+#  $types->{$id}{$part}      .= "v"; # so we know what to tell sender
+#  return 1;
+#}
+
 sub ProcessSymScanEngineOutput {
   my($line, $infections, $types, $BaseDir, $Name) = @_;
   my($logout, $virusid, $virusname, $filename, $action, $shortname);
