@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ZMailer.pm,v 1.1.2.33 2004/12/18 12:25:47 jkf Exp $
+#   $Id: ZMailer.pm,v 1.1.2.37 2005/03/27 18:21:33 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,11 @@
 #      SO17 1BJ
 #      United Kingdom
 #
-
+#   The authors (Leonardo Helman & Mariano Absatz) can be contacted
+#   by email at
+#      MailScanner-devel@pert.com.ar
+#      Pert Consultores
+#      Argentina
 
 package MailScanner::Sendmail;
 
@@ -41,7 +45,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.33 $, 10;
+$VERSION = substr q$Revision: 1.1.2.37 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -321,8 +325,18 @@ sub ReadQf {
       if ($Line =~ /^Subject:\s*(\S.*)?$/i) {
         $message->{subject} = $1;
       }
+      # JKF Testing code, remove before production release
+      #if ($Line =~ /^X-ClientIP:/i) {
+      #  $message->{clientip} = $1 if $Line =~ /(\d+\.\d+\.\d+\.\d+)/;
+      #  $IPFound = 1;
+      #}
+      # JKF End of testing code
     }
   }
+
+  # Decode the ISO encoded Subject line
+  $message->{subject} = MIME::WordDecoder::unmime($message->{subject});
+
   # Every zmailer file should at least define the sender, 1 recipient and the
   # IP address. Everything else is optional, and is preserved as
   # MailScanner may not understand all the types of line.
@@ -396,6 +410,28 @@ sub DeleteHeader {
   }
 }
 
+sub UniqHeader {
+  my($this, $message, $key) = @_;
+
+  my($linenum, $foundat);
+  for ($linenum=0; $linenum<@{$message->{metadata}}; $linenum++) {
+    last if $message->{metadata}[$linenum] =~ /^env-end$|^env-eof$/i;
+  }
+  $foundat = -1;
+  for ($linenum++; $linenum<@{$message->{metadata}}; $linenum++) {
+    next unless $message->{metadata}[$linenum] =~ /^$key/i;
+    # Have found the right line
+    ($foundat = $linenum), next if $foundat == -1;
+
+    splice(@{$message->{metadata}}, $linenum, 1);
+    # Now delete the continuation lines
+    while($message->{metadata}[$linenum] =~ /^\s/) {
+      splice(@{$message->{metadata}}, $linenum, 1);
+    }
+    $linenum--; # Allow for 2 neighbouring instances of $key
+  }
+}
+
 # REVISO LEOH
 sub ReplaceHeader {
   my($this, $message, $key, $newvalue) = @_;
@@ -829,7 +865,8 @@ sub CreateBatch {
       $batch->{messages}{"$id"} = $newmessage;
       $batchempty = 0;
 
-      if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/) {
+      if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
+          MailScanner::Config::Value("dangerscan", $newmessage) =~ /1/) {
         $newmessage->NeedsScanning(1);
         $DirtyMsgs++;
         $DirtyBytes += $newmessage->{size};
